/*
  stage00.c 

  Notice: <<<...>>> is same comment in Japanese (SHIFT JIS code).
*/

#include <assert.h>
#include <nusys.h>
#include "main.h"
#include "graphic.h"
#include "teapi.h"

/* Display X position */
/* <<< \ʒuw >>> */
static float triPos_x;
/* Display Y position */
/* <<< \ʒux >>> */
static float triPos_y;

void shadetri(Dynamic* dynamicp);

/* Initialize Stage 0 */
/* <<< Xe[WȌ >>> */
void initStage00(void)
{
  triPos_x = 0.0;
  triPos_y = 0.0;
#if	1
  teInit();
#endif
}

/* Make display list and start task */
/* <<< fBXvCXgA^XNN >>> */
void makeDL00(void)
{
  Dynamic* dynamicp;

  /* Display list buffer settings */
  /* <<< fBXvCXgobt@̎w >>> */
  dynamicp = &gfx_dynamic[gfx_gtask_no];
  glistp = &gfx_glist[gfx_gtask_no][0];

  /* Initialize RCP */
  /* <<< RCP̏ >>> */
  gfxRCPInit();

  /* Clear frame buffer and Z buffer */
  /* <<< t[obt@Ƃyobt@̃NA >>> */
  gfxClearCfb();

  /* projection,modeling matrix set */
  guOrtho(&dynamicp->projection,
	  -(float)SCREEN_WD/2.0F, (float)SCREEN_WD/2.0F,
	  -(float)SCREEN_HT/2.0F, (float)SCREEN_HT/2.0F,
	  1.0F, 10.0F, 1.0F);
  guRotate(&dynamicp->modeling, 0.0F, 0.0F, 0.0F, 1.0F);
  guTranslate(&dynamicp->translate, triPos_x, triPos_y, 0.0F);

#if	1
	/* Draw background */
	/* <<< wi̕` >>> */
	glistp = teDrawBackground(glistp);
	/* Paste texture on square shape */
	/* <<< `ɃeNX`\ >>> */
	glistp = teDrawTextures(glistp);
	/* Draw menus */
	/* <<< j[` >>> */
	glistp = teDrawMenu(glistp, 2, 2);
#else
  /* Draw square */
  /* <<< `` >>> */
  shadetri(dynamicp);
#endif

  gDPFullSync(glistp++);
  gSPEndDisplayList(glistp++);

  assert((glistp - gfx_glist[gfx_gtask_no]) < GFX_GLIST_LEN);

  /* Start task, and swap display buffer */
  /* <<< ^XNNA\obt@؂ւ܂B >>> */
	if (teMenu_disp) {
		/* Menu display time */
		/* <<< j[\ >>> */
		nuGfxTaskStart(&gfx_glist[gfx_gtask_no][0],
			(s32)(glistp - gfx_glist[gfx_gtask_no]) * sizeof (Gfx),
			NU_GFX_UCODE_F3DEX2 , NU_SC_NOSWAPBUFFER);
#if	1
		/* If nothing drawn, SWAPBUFFER not performed? Problem with nusys/os? */
		/* <<< `悳ĂȂSWAPBUFFERȂHnusys/os̕sH >>> */
		/* Change location of text display */
		/* <<< \ʒu̕ύX >>> */
		nuDebConTextPos(0,0,0);
		/* Display text */
		/* <<< ̕\ >>> */
		/* Space and line feed didn't work, so used "'". */
		/* <<< 󔒂sł͑ʖڂł̂łEEEu'vŁB >>> */
		nuDebConPutc(0, '\'');
#endif
		/* Draw text to frame buffer */
		/* <<< t[obt@ɕ` >>> */
		nuDebConDisp(NU_SC_NOSWAPBUFFER);
		/* Add to performance meter draw list */
		/* <<< ptH[}X[^[̕`惊Xg̒ǉ >>> */
		nuDebTaskPerfBar1(1, 200, NU_SC_SWAPBUFFER);
	} else {
		/* No menu */
		/* <<< j[ >>> */
		nuGfxTaskStart(&gfx_glist[gfx_gtask_no][0],
			(s32)(glistp - gfx_glist[gfx_gtask_no]) * sizeof (Gfx),
			NU_GFX_UCODE_F3DEX2 , NU_SC_SWAPBUFFER);
	}
  /* Swap display list buffer */
  /* <<< fBXvCXgobt@̐؂ւ >>> */
  gfx_gtask_no ^= 1;
}


/* Advance game process for Stage 0 */
/* <<< Xe[WOp̃Q[is >>> */
void updateGame00(void)
{  
  /* Read Controller 1 data */
  /* <<< Rg[P̃f[^ǎ >>> */
  nuContDataGetEx(contdata,0);

  /* Change display position according to Control Stick data */
  /* <<< XeBbNf[^ɂ\ʒuύX >>> */
  triPos_x = contdata->stick_x;
  triPos_y = contdata->stick_y;
#if	1
  /* Controller-based menu manipulation process */
  /* <<< Rg[[ɂ郁j[쏈 >>> */
  teControl(contdata);
#else
  /* Move to Stage 1 after pressing Start button */
  /* <<< X^[g{^ꂽAXe[WPɈڍs >>> */
  if(contdata[0].trigger & START_BUTTON){
    /* Remove callback function */
    /* <<< R[obN֐菜B >>> */
    nuGfxFuncRemove();
    /* Indicate next stage to Main */
    /* <<< ̃Xe[WCɎw >>> */
    stage = 1;
  }
#endif
}

/* Vertex coordinates */
/* <<< _̍W >>> */
static Vtx shade_vtx[] =  {
        {        -64,  64, -5, 0, 0, 0, 0, 0xff, 0, 0xff	},
        {         64,  64, -5, 0, 0, 0, 0, 0, 0, 0xff	},
        {         64, -64, -5, 0, 0, 0, 0, 0, 0xff, 0xff	},
        {        -64, -64, -5, 0, 0, 0, 0xff, 0, 0, 0xff	},
};

/* Compose a square */
/* <<< ` >>> */
void shadetri(Dynamic* dynamicp)
{
  gSPMatrix(glistp++,OS_K0_TO_PHYSICAL(&(dynamicp->projection)),
		G_MTX_PROJECTION|G_MTX_LOAD|G_MTX_NOPUSH);
  gSPMatrix(glistp++,OS_K0_TO_PHYSICAL(&(dynamicp->modeling)),
		G_MTX_MODELVIEW|G_MTX_LOAD|G_MTX_NOPUSH);
  gSPMatrix(glistp++,OS_K0_TO_PHYSICAL(&(dynamicp->translate)),
		G_MTX_MODELVIEW|G_MTX_MUL|G_MTX_NOPUSH);

  gSPVertex(glistp++,&(shade_vtx[0]),4, 0);

  gDPPipeSync(glistp++);
  gDPSetCycleType(glistp++,G_CYC_1CYCLE);
  gDPSetRenderMode(glistp++,G_RM_AA_OPA_SURF, G_RM_AA_OPA_SURF2);
  gSPClearGeometryMode(glistp++,0xFFFFFFFF);
  gSPSetGeometryMode(glistp++,G_SHADE| G_SHADING_SMOOTH);

  gSP2Triangles(glistp++,0,1,2,0,0,2,3,0);
}

